#include "stdio.h"
#include "string.h"
#include "assert.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"
#include "SDL_mixer.h"

#include "BList.h"

#include "samples.h"
#include "animscript.h"
#include "conditionscript.h"
#include "actionscript.h"
#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "tiles.h"

#include "auxiliar.h"
#include "MOG2_auxiliar.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

OBJECT::OBJECT(void)
{
	int i,j;

	name=0;
	ntiles=0;
	tiles=0;
	link_x=0;
	link_y=0;
	nlinks=0;
	nparts=1;
	link_part=new int[1];
	link_a=new int[1];
	link_b=new int[1];
	link_part[0]=-1;
	link_a[0]=0;
	link_b[0]=0;

	for(j=0;j<MAX_PARTS;j++) {
		default_tile[j]=0;
		part_precedence[j]=0;
	} /* for */ 

	constitution=CONS_NONE;
	control=CTRL_STANDARD;
	active_flag=false;
	pushable_flag=false;
	multiroom_flag=false;
	collision_flag=false;

	for(i=0;i<CPOS_NSTATES;i++) {
		for(j=0;j<MAX_PARTS;j++) {
			state_anim[j][i]=new AnimSCRIPT();
		} /* for */ 
		state_sample[i]=0;
		state_sample_command[i]=0;
	    state_sample_command_parameter[i]=0;
		state_sample_volume[i]=100;
	} /* for */ 
	stair_states=false;
	jump_states=false;

	variable_jumping=true;
	controllable_jump=true;
	jumping_ctnt=11;
	gravity_ctnt=1;
	jumping_key=-3;
	walking_speed=(TILESIZE/4)*100;
	ladder_up_speed=(TILESIZE/8)*100;
	ladder_down_speed=(TILESIZE/8)*100;
	stop_at_each_floor=true;

	n_states=0;
	n_s_states=0;
	for(j=0;j<MAX_PARTS;j++) {
		g_state_anim[j]=0;
		g_s_state_anim[j]=0;
	} /* for */ 
	g_state_description=0;
	g_state_action=0;

	g_s_state_description=0;
	g_s_state_action=0;
	g_s_state_condition=0;
	g_s_state_exclusiveness=0;
	g_s_state_priority=0;

	init_state=0;
	init_energy=1;
	experience=0;
	damage=1;
	damage_type=0;

	/* Drawing_mode: */ 
	dm_drawing_mode=0;
	dm_color1_r=dm_color1_g=dm_color1_b=255;
    dm_color1_a=-1;
	dm_color2_r=dm_color2_g=dm_color2_b=0;
    dm_color2_a=-1;
	dm_factor=100;

	scale=100;
	angle=0;

	text_initialization();

	gw_dx=512;
	gw_dy=320;
	gw_scale=100;
	gw_min_hs_x=50;
	gw_min_hs_y=60;
	gw_max_hs_x=50;
	gw_max_hs_y=60;

	menu_dx=128;
	menu_dy=128;
	menu_title=0;
	menu_appearance_flags=0;
	menu_entry_effect_flags=0;
	menu_blink_timmer=0;
	menu_zoom_factor=0;
	menu_appear_factor=0;

	menu_text_r2=255;
	menu_text_g2=0;
	menu_text_b2=0;
	menu_text_a2=255;

} /* OBJECT::OBJECT */ 


OBJECT::OBJECT(char *n)
{
	int i,j;

	name=new char[strlen(n)+1];
	strcpy(name,n);
	ntiles=0;
	tiles=0;
	link_x=0;
	link_y=0;
	nlinks=0;
	nparts=1;
	link_part=new int[1];
	link_a=new int[1];
	link_b=new int[1];
	link_part[0]=-1;
	link_a[0]=0;
	link_b[0]=0;

	for(j=0;j<MAX_PARTS;j++) {
		default_tile[j]=0;
		part_precedence[j]=0;
	} /* for */ 

	constitution=CONS_NONE;
	control=CTRL_STANDARD;
	active_flag=false;
	pushable_flag=false;
	multiroom_flag=false;
	collision_flag=false;

	for(i=0;i<CPOS_NSTATES;i++) {
		for(j=0;j<MAX_PARTS;j++) {
			state_anim[j][i]=new AnimSCRIPT();
		} /* for */ 
		state_sample[i]=0;
		state_sample_command[i]=0;
	    state_sample_command_parameter[i]=0;
		state_sample_volume[i]=100;
	} /* for */ 
	stair_states=false;
	jump_states=false;

	variable_jumping=true;
	controllable_jump=true;
	jumping_ctnt=11;
	gravity_ctnt=1;
	jumping_key=-3;
	walking_speed=(TILESIZE/4)*100;
	ladder_up_speed=(TILESIZE/8)*100;
	ladder_down_speed=(TILESIZE/8)*100;
	stop_at_each_floor=true;

	n_states=0;
	n_s_states=0;
	for(j=0;j<MAX_PARTS;j++) {
		g_state_anim[j]=0;
		g_s_state_anim[j]=0;
	} /* for */ 
	g_state_description=0;
	g_state_action=0;

	g_s_state_description=0;
	g_s_state_action=0;
	g_s_state_condition=0;
	g_s_state_exclusiveness=0;
	g_s_state_priority=0;

	init_state=0;
	init_energy=1;
	experience=0;
	damage=1;
	damage_type=0;
        
	/* Drawing_mode: */ 
	dm_drawing_mode=0;
	dm_color1_r=dm_color1_g=dm_color1_b=255;
    dm_color1_a=-1;
	dm_color2_r=dm_color2_g=dm_color2_b=0;
    dm_color2_a=-1;
	dm_factor=100;

	scale=100;
	angle=0;

	text_initialization();

	gw_dx=512;
	gw_dy=320;
	gw_scale=100;
	gw_min_hs_x=50;
	gw_min_hs_y=60;
	gw_max_hs_x=50;
	gw_max_hs_y=60;

	menu_dx=128;
	menu_dy=128;
	menu_title=0;
	menu_appearance_flags=0;
	menu_entry_effect_flags=0;
	menu_blink_timmer=0;
	menu_zoom_factor=0;
	menu_appear_factor=0;

	menu_text_r2=255;
	menu_text_g2=0;
	menu_text_b2=0;
	menu_text_a2=255;

} /* OBJECT::OBJECT */ 


OBJECT::OBJECT(char *n,OBJECT *o)
{
	List<VARIABLE> l;
	VARIABLE *v;
	int i,j;

	name=new char[strlen(n)+1];
	strcpy(name,n);
	ntiles=o->ntiles;
	tiles=new TILE *[ntiles];
	link_x=new int *[ntiles];
	link_y=new int *[ntiles];
	nlinks=new int[ntiles];
	for(i=0;i<ntiles;i++) {
		tiles[i]=o->tiles[i];
		link_x[i]=new int[MAX_LINKS];
		link_y[i]=new int[MAX_LINKS];
		nlinks[i]=o->nlinks[i];
		for(j=0;j<nlinks[i];j++) {
			link_x[i][j]=o->link_x[i][j];
			link_y[i][j]=o->link_y[i][j];
		} /* for */ 
	} /* for */ 

	for(j=0;j<MAX_PARTS;j++) {
		default_tile[j]=o->default_tile[j];
		part_precedence[j]=o->part_precedence[j];
	} /* for */ 

	nparts=o->nparts;
	link_part=new int[nparts];
	link_a=new int[nparts];
	link_b=new int[nparts];
	for(i=0;i<nparts;i++) {
		link_part[i]=o->link_part[i];
		link_a[i]=o->link_a[i];
		link_b[i]=o->link_b[i];
	} /* for */ 

	constitution=o->constitution;
	control=o->control;
	active_flag=o->active_flag;
	pushable_flag=o->pushable_flag;
	multiroom_flag=o->multiroom_flag;
	collision_flag=o->collision_flag;

	for(i=0;i<CPOS_NSTATES;i++) {
		for(j=0;j<MAX_PARTS;j++) {
			state_anim[j][i]=new AnimSCRIPT(*o->state_anim[j][i]);
		} /* for */ 
		state_sample[i]=o->state_sample[i];
		state_sample_command[i]=o->state_sample_command[i];
	    state_sample_command_parameter[i]=o->state_sample_command_parameter[i];
		state_sample_volume[i]=o->state_sample_volume[i];
	} /* for */ 
	stair_states=o->stair_states;
	jump_states=o->jump_states;

	walking_speed=o->walking_speed;
	jumping_ctnt=o->jumping_ctnt;
	gravity_ctnt=o->gravity_ctnt;
	jumping_key=o->jumping_key;
	variable_jumping=o->variable_jumping;
	controllable_jump=o->controllable_jump;
	ladder_up_speed=o->ladder_up_speed;
	ladder_down_speed=o->ladder_down_speed;
	stop_at_each_floor=o->stop_at_each_floor;

	l.Instance(o->object_variables);
	l.Rewind();
	while(l.Iterate(v)) {
		object_variables.Add(new VARIABLE(*v));
	} /* while */ 


	n_states=o->n_states;
	n_s_states=o->n_s_states;

	for(j=0;j<MAX_PARTS;j++) {
		g_state_anim[j]=new AnimSCRIPT *[n_states];
		g_s_state_anim[j]=new AnimSCRIPT *[n_s_states];
	} /* for */ 
	g_state_description=new char *[n_states];
	g_state_action=new ActionSCRIPT *[n_states];

	g_s_state_description=new char *[n_s_states];
	g_s_state_action=new ActionSCRIPT *[n_s_states];
	g_s_state_condition=new ConditionSCRIPT *[n_s_states];
	g_s_state_exclusiveness=new bool [n_s_states];
	g_s_state_priority=new int[n_s_states];
	for(i=0;i<n_states;i++) {
		for(j=0;j<MAX_PARTS;j++) {
			g_state_anim[j][i]=new AnimSCRIPT(*o->g_state_anim[j][i]);
		} /* for */ 
		g_state_description[i]=new char[strlen(o->g_state_description[i])+1];
		strcpy(g_state_description[i],o->g_state_description[i]);
		g_state_action[i]=new ActionSCRIPT(*o->g_state_action[i]);
	} /* for */ 

	for(i=0;i<n_s_states;i++) {
		for(j=0;j<MAX_PARTS;j++) {
			g_s_state_anim[j][i]=new AnimSCRIPT(*o->g_s_state_anim[j][i]);
		} /* for */ 
		g_s_state_description[i]=new char[strlen(o->g_s_state_description[i])+1];
		strcpy(g_s_state_description[i],o->g_s_state_description[i]);
		g_s_state_action[i]=new ActionSCRIPT(*o->g_s_state_action[i]);
		g_s_state_condition[i]=new ConditionSCRIPT(*o->g_s_state_condition[i]);
		g_s_state_exclusiveness[i]=o->g_s_state_exclusiveness[i];
		g_s_state_priority[i]=o->g_s_state_priority[i];
	} /* for */ 

	init_state=o->init_state;
	init_energy=o->init_energy;
	experience=o->experience;
	damage=o->damage;
	if (o->damage_type!=0) {
		damage_type=new char[strlen(o->damage_type)+1];
		strcpy(damage_type,o->damage_type);
	} else {
		damage_type=0;
	} /* if */ 
        
	/* Drawing_mode: */ 
	dm_drawing_mode=o->dm_drawing_mode;
	dm_factor=o->dm_factor;
	dm_color1_r=o->dm_color1_r;
	dm_color1_g=o->dm_color1_g;
	dm_color1_b=o->dm_color1_b;
	dm_color1_a=o->dm_color1_a;
	dm_color2_r=o->dm_color2_r;
	dm_color2_g=o->dm_color2_g;
	dm_color2_b=o->dm_color2_b;
	dm_color2_a=o->dm_color2_a;

	scale=o->scale;
	angle=o->angle;

	text_initialization(o);

	gw_dx=o->gw_dx;
	gw_dy=o->gw_dy;
	gw_scale=o->gw_scale;
	gw_min_hs_x=o->gw_min_hs_x;
	gw_min_hs_y=o->gw_min_hs_y;
	gw_max_hs_x=o->gw_max_hs_x;
	gw_max_hs_y=o->gw_max_hs_y;

	menu_dx=o->menu_dx;
	menu_dy=o->menu_dy;
	if (o->menu_title!=0) {
		menu_title=new char[strlen(o->menu_title)+1];
		strcpy(menu_title,o->menu_title);
	} else {
		menu_title=0;
	} /* if */ 
	menu_appearance_flags=o->menu_appearance_flags;
	menu_entry_effect_flags=o->menu_entry_effect_flags;
	menu_blink_timmer=o->menu_blink_timmer;
	menu_zoom_factor=o->menu_zoom_factor;
	menu_appear_factor=o->menu_appear_factor;

	menu_text_r2=255;
	menu_text_g2=0;
	menu_text_b2=0;
	menu_text_a2=255;

} /* OBJECT::OBJECT */ 


OBJECT::~OBJECT(void)
{
	int i,j;

	if (name!=0) delete []name;
	name=0;
	if (tiles!=0) delete []tiles;
	tiles=0;

	for(i=0;i<ntiles;i++) {
		if (link_x[i]!=0) delete []link_x[i];
		link_x[i]=0;
		if (link_y[i]!=0) delete []link_y[i];
		link_y[i]=0;
	} /* for */ 
	delete []link_x;
	link_x=0;
	delete []link_y;
	link_y=0;
	delete []nlinks;
	nlinks=0;

	delete []link_part;
	link_part=0;
	delete []link_a;
	link_a=0;
	delete []link_b;
	link_b=0;

	for(i=0;i<CPOS_NSTATES;i++) {
		for(j=0;j<MAX_PARTS;j++) {
			delete state_anim[j][i];
		} /* for */ 
	} /* for */ 


	if (n_states!=0) {
		for(i=0;i<n_states;i++) {
			for(j=0;j<MAX_PARTS;j++) delete g_state_anim[j][i];
			for(j=0;j<MAX_PARTS;j++) g_state_anim[j][i]=0;
			delete g_state_action[i];
			g_state_action[i]=0;
			delete []g_state_description[i];
			g_state_description[i]=0;
		} /* for */ 
		for(j=0;j<MAX_PARTS;j++) delete []g_state_anim[j];
		delete []g_state_action;
		delete []g_state_description;
	} /* if */ 

	if (n_s_states!=0) {
		for(i=0;i<n_s_states;i++) {
			for(j=0;j<MAX_PARTS;j++) delete g_s_state_anim[j][i];
			for(j=0;j<MAX_PARTS;j++) g_s_state_anim[j][i]=0;

			delete g_s_state_action[i];
			g_s_state_action[i]=0;
			delete g_s_state_condition[i];
			g_s_state_condition[i]=0;
			delete []g_s_state_description[i];
			g_s_state_description[i]=0;
		} /* for */ 
		for(j=0;j<MAX_PARTS;j++) delete []g_s_state_anim[j];
		delete []g_s_state_action;
		delete []g_s_state_condition;
		delete []g_s_state_exclusiveness;
		delete []g_s_state_priority;
		delete []g_s_state_description;
	} /* if */ 

	if (damage_type!=0) delete []damage_type;

	/* Text: */ 
	if (text_text!=0) delete []text_text;
	if (text_font!=0) delete []text_font;
	text_text=0;
	text_font=0;

	if (menu_title!=0) delete []menu_title;
	menu_title=0;

} /* OBJECT::~OBJECT */ 


bool OBJECT::save(FILE *fp,MOG2 *game)
{
	int i,j,t;
	bool found;
	char *tmp;
	
	encode_string(name,&tmp);
	fprintf(fp,"NAME %s\n",tmp);
	delete []tmp;

	fprintf(fp,"%i\n",ntiles);

	for(i=0;i<ntiles;i++) {
		found=false;
		for(j=0;!found && j<TILECATEGORIES;j++) {
			t=game->tiles[j].SearchObjRef(tiles[i]);
			if (t!=-1) found=true;
		} /* for */ 
		fprintf(fp,"%i %i ",j-1,t);

		/* links: */ 
		fprintf(fp,"%i\n",nlinks[i]);
		for(j=0;j<nlinks[i];j++) {
			fprintf(fp,"%i %i\n",link_x[i][j],link_y[i][j]);
		} /* for */ 
	} /* for */ 

	fprintf(fp,"%i\n",nparts);
	for(i=0;i<nparts;i++) {
		fprintf(fp,"%i %i %i %i %i\n",link_part[i],link_a[i],link_b[i],default_tile[i],part_precedence[i]);
	} /* for */ 

	fprintf(fp,"%i\n",constitution);
	fprintf(fp,"%i\n",control);

	/* character values: */ 
	fprintf(fp,"%i %i\n",(stair_states ? 1 : 0),(jump_states ? 1 : 0));
	for(i=0;i<CPOS_NSTATES;i++) {
		int k;
		for(j=0;j<nparts;j++) state_anim[j][i]->save(fp);

		k=game->samples.SearchObjRef(state_sample[i]);

		fprintf(fp,"%i %i %i %i\n",state_sample_command[i],k,state_sample_command_parameter[i],state_sample_volume[i]);
	} /* for */ 
	fprintf(fp,"%i %i %i %i %i %i %i %i %i\n",walking_speed,ladder_up_speed,ladder_down_speed,jumping_ctnt,gravity_ctnt,jumping_key,(variable_jumping ? 1:0),(controllable_jump ? 1:0),(stop_at_each_floor ? 1:0));

	/* general values: */ 
	fprintf(fp,"%i %i %i %i\n",(active_flag ? 1 : 0),(pushable_flag ? 1 : 0),(multiroom_flag ? 1 : 0),(collision_flag ? 1 : 0));
	fprintf(fp,"%i\n",n_states);
	for(i=0;i<n_states;i++) {
		for(j=0;j<nparts;j++) g_state_anim[j][i]->save(fp);
		g_state_action[i]->save(fp);
		encode_string(g_state_description[i],&tmp);
		fprintf(fp,"%s\n",tmp);
		delete []tmp;
	} /* for */ 

	fprintf(fp,"%i\n",n_s_states);
	for(i=0;i<n_s_states;i++) {
		for(j=0;j<nparts;j++) g_s_state_anim[j][i]->save(fp);
		g_s_state_action[i]->save(fp);
		g_s_state_condition[i]->save(fp);
		fprintf(fp,"%i\n",(g_s_state_exclusiveness[i] ? 1 : 0));
		fprintf(fp,"%i\n",g_s_state_priority[i]);
		encode_string(g_s_state_description[i],&tmp);
		fprintf(fp,"%s\n",tmp);
		delete []tmp;
	} /* for */ 

	/* Variables: */ 
	fprintf(fp,"%i %i %i %i\n",init_state,init_energy,experience,damage);
	if (damage_type!=0) {
		encode_string(damage_type,&tmp);
		fprintf(fp,"DAMAGE %s\n",tmp);
		delete []tmp;
	} else {
		fprintf(fp,"NODAMAGE\n");
	} /* if */ 

	/* Text variables: */ 
	if (text_text!=0) {
		encode_string(text_text,&tmp);
		fprintf(fp,"TEXT %s\n",tmp);
		delete []tmp;
	} else {
		fprintf(fp,"NOTEXT\n");
	} /* if */ 
	if (text_font!=0) {
		encode_string(text_font,&tmp);
		fprintf(fp,"FONT %s\n",tmp);
		delete []tmp;
	} else {
		fprintf(fp,"NOFONT\n");
	} /* if */ 
	fprintf(fp,"%i\n",text_size);
	fprintf(fp,"%i\n",text_spacing);
	fprintf(fp,"%i %i %i %i\n",text_r,text_g,text_b,text_a);
	fprintf(fp,"%i %i\n",text_frame,text_frame_width);
	fprintf(fp,"%i %i %i %i\n",text_frame_r1,text_frame_g1,text_frame_b1,text_frame_a1);
	fprintf(fp,"%i %i %i %i\n",text_frame_r2,text_frame_g2,text_frame_b2,text_frame_a2);
	fprintf(fp,"%i\n",text_frame_radius);
	fprintf(fp,"%i %i\n",text_frame_point_x,text_frame_point_y);
	fprintf(fp,"%i\n",text_h_align,text_v_align);
	fprintf(fp,"%i\n",text_max_w);

	/* gw variables: */ 
	fprintf(fp,"%i %i %i %i %i %i %i\n",gw_dx,gw_dy,gw_scale,gw_min_hs_x,gw_min_hs_y,gw_max_hs_x,gw_max_hs_y);

	/* menu variables: */ 
	fprintf(fp,"%i %i %i %i %i %i %i %i %i %i %i\n",
			menu_dx,menu_dy,menu_appearance_flags,menu_entry_effect_flags,
			menu_blink_timmer,menu_zoom_factor,menu_appear_factor, 
			menu_text_r2,menu_text_g2,menu_text_b2,menu_text_a2);
	if (menu_title!=0) {
		encode_string(menu_title,&tmp);
		fprintf(fp,"TITLE %s\n",tmp);
		delete []tmp;
	} else {
		fprintf(fp,"NOTITLE\n");
	} /* if */ 

	/* General variables: */ 
	fprintf(fp,"VARIABLES %i\n",object_variables.Length());
	{
		List<VARIABLE> l;
		VARIABLE *v;

		l.Instance(object_variables);
		l.Rewind();
		while(l.Iterate(v)) v->save(fp);
	}
        
    /* Drawing mode: */ 
    fprintf(fp,"%i %i %i %i %i %i %i %i %i %i\n",dm_drawing_mode,dm_factor,
            dm_color1_r,dm_color1_g,dm_color1_b,dm_color1_a,
            dm_color2_r,dm_color2_g,dm_color2_b,dm_color2_a);

	/* Default scale/angle: */ 
	fprintf(fp,"%i %i\n",scale,angle);

	return true;
} /* OBJECT::save */ 


bool OBJECT::load(FILE *fp,MOG2 *game)
{
	int i,j,k;
	char tmp[80];

	if (name!=0) delete []name;
	name=0;
	if (tiles!=0) delete []tiles;
	tiles=0;

	for(i=0;i<ntiles;i++) {
		if (link_x[i]!=0) delete []link_x[i];
		link_x[i]=0;
		if (link_y[i]!=0) delete []link_y[i];
		link_y[i]=0;
	} /* for */ 
	delete []link_x;
	link_x=0;
	delete []link_y;
	link_y=0;
	delete []nlinks;
	nlinks=0;

	delete []link_part;
	link_part=0;
	delete []link_a;
	link_a=0;
	delete []link_b;
	link_b=0;

	if (1!=fscanf(fp,"%s ",tmp)) return false;
	if (name!=0) delete name;
	if (!load_string(fp,&name)) return false;

	if (1!=fscanf(fp,"%i",&ntiles)) return false;
	if (tiles!=0) delete tiles;
	tiles=new TILE *[ntiles];
	link_x=new int *[ntiles];
	link_y=new int *[ntiles];
	nlinks=new int[ntiles];

	for(i=0;i<ntiles;i++) {
		int j,t;
		if (2!=fscanf(fp,"%i %i",&j,&t)) return false;

		tiles[i]=game->tiles[j][t];

		/* links: */ 
		link_x[i]=new int[MAX_LINKS];
		link_y[i]=new int[MAX_LINKS];
		if (1!=fscanf(fp,"%i",&nlinks[i])) return false;
		for(j=0;j<nlinks[i];j++) {
			if (2!=fscanf(fp,"%i %i",&link_x[i][j],&link_y[i][j])) return false;
		} /* for */ 
	} /* for */ 

	if (1!=fscanf(fp,"%i",&nparts)) return false;
	link_part=new int[nparts];
	link_a=new int[nparts];
	link_b=new int[nparts];
	for(i=0;i<nparts;i++) {
		if (5!=fscanf(fp,"%i %i %i %i %i",&link_part[i],&link_a[i],&link_b[i],&default_tile[i],&part_precedence[i])) return false;
	} /* for */ 

	if (1!=fscanf(fp,"%i",&constitution)) return false;
	if (1!=fscanf(fp,"%i",&control)) return false;
	
	/* character values: */ 
	if (1!=fscanf(fp,"%i",&i)) return false;
	stair_states=(i==1 ? true : false);
	if (1!=fscanf(fp,"%i",&i)) return false;
	jump_states=(i==1 ? true : false);
	for(i=0;i<CPOS_NSTATES;i++) {
		int k;
		for(j=0;j<nparts;j++) 		
			if (!state_anim[j][i]->load(fp,game)) return false;
		if (4!=fscanf(fp,"%i %i %i %i",&state_sample_command[i],&k,&state_sample_command_parameter[i],&state_sample_volume[i])) return false;
		if (k==-1) {
			state_sample[i]=0;
		} else {
			state_sample[i]=game->samples[k];
		} /* if */ 
	} /* for */ 
	if (9!=fscanf(fp,"%i %i %i %i %i %i %i %i %i",&walking_speed,&ladder_up_speed,&ladder_down_speed,&jumping_ctnt,&gravity_ctnt,&jumping_key,&i,&j,&k)) return false;
	variable_jumping=(i==1 ? true:false);
	controllable_jump=(j==1 ? true:false);
	stop_at_each_floor=(k==1 ? true:false);

	/* general values: */ 
	if (1!=fscanf(fp,"%i",&i)) return false;
	active_flag=(i==1 ? true : false);
	if (1!=fscanf(fp,"%i",&i)) return false;
	pushable_flag=(i==1 ? true : false);
	if (1!=fscanf(fp,"%i",&i)) return false;
	multiroom_flag=(i==1 ? true : false);
	if (1!=fscanf(fp,"%i",&i)) return false;
	collision_flag=(i==1 ? true : false);

	if (1!=fscanf(fp,"%i",&n_states)) return false;
	if (n_states>0) {
		for(j=0;j<MAX_PARTS;j++) g_state_anim[j]=new AnimSCRIPT *[n_states];
		g_state_action=new ActionSCRIPT *[n_states];
		g_state_description=new char *[n_states];
		for(i=0;i<n_states;i++) {
			for(j=0;j<MAX_PARTS;j++) g_state_anim[j][i]=0;
			g_state_action[i]=0;
			g_state_description[i]=0;
		} /* for */ 
		for(i=0;i<n_states;i++) {
			for(j=0;j<MAX_PARTS;j++) g_state_anim[j][i]=new AnimSCRIPT();
			for(j=0;j<nparts;j++) 
				if (!g_state_anim[j][i]->load(fp,game)) return false;

			g_state_action[i]=new ActionSCRIPT();
			if (!g_state_action[i]->load(fp,game)) return false;

			if (g_state_description[i]!=0) delete []g_state_description[i];
			if (!load_string(fp,&g_state_description[i])) return false;
		} /* for */ 
	} else {
		for(j=0;j<MAX_PARTS;j++) g_state_anim[j]=0;
		g_state_action=0;
		g_state_description=0;
	} /* if */ 
	
	if (1!=fscanf(fp,"%i",&n_s_states)) return false;
	if (n_s_states>0) {
		for(j=0;j<MAX_PARTS;j++) g_s_state_anim[j]=new AnimSCRIPT *[n_s_states];
		g_s_state_action=new ActionSCRIPT *[n_s_states];
		g_s_state_condition=new ConditionSCRIPT *[n_s_states];
		g_s_state_exclusiveness=new bool[n_s_states];
		g_s_state_priority=new int[n_s_states];
		g_s_state_description=new char *[n_s_states];
		for(i=0;i<n_s_states;i++) {
			for(j=0;j<MAX_PARTS;j++) g_s_state_anim[j][i]=0;
			g_s_state_action[i]=0;
			g_s_state_condition[i]=0;
			g_s_state_exclusiveness[i]=false;
			g_s_state_priority[i]=0;
			g_s_state_description[i]=0;
		} /* for */ 
		for(i=0;i<n_s_states;i++) {
			for(j=0;j<MAX_PARTS;j++) g_s_state_anim[j][i]=new AnimSCRIPT();
			for(j=0;j<nparts;j++) 
				if (!g_s_state_anim[j][i]->load(fp,game)) return false;

			g_s_state_action[i]=new ActionSCRIPT();
			if (!g_s_state_action[i]->load(fp,game)) return false;

			g_s_state_condition[i]=new ConditionSCRIPT();
			if (!g_s_state_condition[i]->load(fp,game)) return false;

			if (1!=fscanf(fp,"%i",&j)) return false;
			g_s_state_exclusiveness[i]=(j==1 ? true : false);

			if (1!=fscanf(fp,"%i",&g_s_state_priority[i])) return false;

			if (g_s_state_description[i]!=0) delete []g_s_state_description[i];
			if (!load_string(fp,&g_s_state_description[i])) return false;
		} /* for */ 
	} else {
		for(j=0;j<MAX_PARTS;j++) g_s_state_anim[j]=0;
		g_s_state_action=0;
		g_s_state_condition=0;
		g_s_state_exclusiveness=0;
		g_s_state_priority=0;
		g_s_state_description=0;
	} /* if */ 

	/* Variables: */ 
	if (4!=fscanf(fp,"%i %i %i %i",&init_state,&init_energy,&experience,&damage)) return false;

	if (1!=fscanf(fp,"%s ",tmp)) return false;
	if (strcmp(tmp,"DAMAGE")==0) {
		if (damage_type!=0) delete []damage_type;
		if (!load_string(fp,&damage_type)) return false;
	} else {
		damage_type=0;
	} /* if */ 

	/* Text variables: */ 

	if (1!=fscanf(fp,"%s ",tmp)) return false;
	if (strcmp(tmp,"TEXT")==0) {
		if (text_text!=0) delete []text_text;
		if (!load_string(fp,&text_text)) return false;
	} else {
		text_text=0;
	} // if 
	if (1!=fscanf(fp,"%s ",tmp)) return false;
	if (strcmp(tmp,"FONT")==0) {
		if (text_font!=0) delete []text_font;
		if (!load_string(fp,&text_font)) return false;
	} else {
		text_font=0;
	} // if 

	if (1!=fscanf(fp,"%i",&text_size)) return false;
	if (1!=fscanf(fp,"%i",&text_spacing)) return false;
	if (4!=fscanf(fp,"%i %i %i %i",&text_r,&text_g,&text_b,&text_a)) return false;
	if (2!=fscanf(fp,"%i %i",&text_frame,&text_frame_width)) return false;
	if (4!=fscanf(fp,"%i %i %i %i",&text_frame_r1,&text_frame_g1,&text_frame_b1,&text_frame_a1)) return false;
	if (4!=fscanf(fp,"%i %i %i %i",&text_frame_r2,&text_frame_g2,&text_frame_b2,&text_frame_a2)) return false;
	if (1!=fscanf(fp,"%i",&text_frame_radius)) return false;
	if (2!=fscanf(fp,"%i %i",&text_frame_point_x,&text_frame_point_y)) return false;
	if (1!=fscanf(fp,"%i",&text_h_align,&text_v_align)) return false;
	if (1!=fscanf(fp,"%i",&text_max_w)) return false;

	/* gw variables: */ 
	if (7!=fscanf(fp,"%i %i %i %i %i %i %i",&gw_dx,&gw_dy,&gw_scale,
											  &gw_min_hs_x,&gw_min_hs_y,&gw_max_hs_x,&gw_max_hs_y)) return false;

	/* menu variables: */ 
	if (11!=fscanf(fp,"%i %i %i %i %i %i %i %i %i %i %i",
				    &menu_dx,&menu_dy,&menu_appearance_flags,&menu_entry_effect_flags,
				    &menu_blink_timmer,&menu_zoom_factor,&menu_appear_factor, 
				    &menu_text_r2,&menu_text_g2,&menu_text_b2,&menu_text_a2)) return false;
	if (1!=fscanf(fp,"%s ",tmp)) return false;
	if (strcmp(tmp,"TITLE")==0) {
		if (menu_title!=0) delete []menu_title;
		if (!load_string(fp,&menu_title)) return false;
	} else {
		menu_title=0;
	} // if 

	/* General variables: */ 
	if (2!=fscanf(fp,"%s %i",tmp,&j)) return false;
	for(i=0;i<j;i++) {
		VARIABLE *v;

		v=new VARIABLE();
		v->load(fp);
		object_variables.Add(v);
	} /* for */ 
        
    /* Drawing mode: */ 
    if (10!=fscanf(fp,"%i %i %i %i %i %i %i %i %i %i\n",&dm_drawing_mode,&dm_factor,
                   &dm_color1_r,&dm_color1_g,&dm_color1_b,&dm_color1_a,
                   &dm_color2_r,&dm_color2_g,&dm_color2_b,&dm_color2_a)) return false;

	/* Default scale/angle: */ 
	if (2!=fscanf(fp,"%i %i\n",&scale,&angle)) return false;

	return true;
} /* OBJECT::load */ 


bool OBJECT::cmp(char *n)
{
	if (strcmp(name,n)==0) return true;

	return false;
} /* OBJECT::cmp */ 


bool OBJECT::cmp(OBJECT *o)
{
	if (strcmp(name,o->name)==0) return true;

	return false;
} /* OBJECT::cmp */ 



void OBJECT::draw(int x,int y,int *tile,int *p_offset_p,SDL_Surface *sfc,SDL_Rect *vp)
{
	int x1,x2,y1,y2;
	int i;
	int dx,dy;
	int order[MAX_PARTS];

	if (vp!=0) get_bbox(x,y,tile,&x1,&y1,&x2,&y2);

	if (vp==0 ||
		(x2>vp->x &&
		 y2>vp->y &&
		 x1<vp->x+vp->w &&
		 y1<vp->y+vp->h)) {
		/* Draw bitmaps: */ 

		get_parts_order(order,p_offset_p);

		for(i=0;i<nparts;i++) {
			if (tile[order[i]]>=0 && tile[order[i]]<ntiles) {
				get_part_position(order[i],tile,&dx,&dy);
				tiles[tile[order[i]]]->draw(x+dx,y+dy,sfc);
			} /* if */ 
		} /* for */ 
	} /* if */ 
} /* OBJECT::draw */ 


void OBJECT::draw_shaded(int x,int y,int *tile,int *p_offset_p,SDL_Surface *sfc,SDL_Rect *vp,int factor,int r,int g,int b,int a)
{
	int x1,x2,y1,y2;
	int i;
	int dx,dy;
	int order[MAX_PARTS];

	if (vp!=0) get_bbox(x,y,tile,&x1,&y1,&x2,&y2);

	if (vp==0 ||
		(x2>vp->x &&
		 y2>vp->y &&
		 x1<vp->x+vp->w &&
		 y1<vp->y+vp->h)) {
		/* Draw bitmaps: */ 
		get_parts_order(order,p_offset_p);

		for(i=0;i<nparts;i++) {
			if (tile[order[i]]>=0 && tile[order[i]]<ntiles) {
				get_part_position(order[i],tile,&dx,&dy);
				tiles[tile[order[i]]]->draw_shaded(x+dx,y+dy,sfc,factor,r,g,b,a);
			} /* if */ 
		} /* for */ 
	} /* if */ 
} /* OBJECT::draw_shaded */ 


void OBJECT::draw_bicolor(int x,int y,int *tile,int *p_offset_p,SDL_Surface *sfc,SDL_Rect *vp,int factor,int r1,int g1,int b1,int a1,int r2,int g2,int b2,int a2)
{
	int x1,x2,y1,y2;
	int i;
	int dx,dy;
	int order[MAX_PARTS];

	if (vp!=0) get_bbox(x,y,tile,&x1,&y1,&x2,&y2);

	if (vp==0 ||
		(x2>vp->x &&
		 y2>vp->y &&
		 x1<vp->x+vp->w &&
		 y1<vp->y+vp->h)) {
		/* Draw bitmaps: */ 

		get_parts_order(order,p_offset_p);

		for(i=0;i<nparts;i++) {
			if (tile[order[i]]>=0 && tile[order[i]]<ntiles) {
				get_part_position(order[i],tile,&dx,&dy);
				tiles[tile[order[i]]]->draw_bicolor(x+dx,y+dy,sfc,factor,r1,g1,b1,a1,r2,g2,b2,a2);
			} /* if */ 
		} /* for */ 
	} /* if */ 
} /* OBJECT::draw_bicolor */ 



void OBJECT::draw_reflection(int x,int y,int *tile,int *p_offset_p,SDL_Surface *sfc,SDL_Rect *vp,int reflection_kh,int reflection_kv,int reflection_oh,int reflection_ov,int reflection_wave_height,int center_x,int center_y,int factor)
{
	int x1,x2,y1,y2;
	int i;
	int dx,dy;
	int order[MAX_PARTS];

	if (vp!=0) get_bbox(x,y,tile,&x1,&y1,&x2,&y2);

	if (vp==0 ||
		(x2>vp->x &&
		 y2>vp->y &&
		 x1<vp->x+vp->w &&
		 y1<vp->y+vp->h)) {
		/* Draw bitmaps: */ 

		get_parts_order(order,p_offset_p);

		for(i=0;i<nparts;i++) {
			if (tile[order[i]]>=0 && tile[order[i]]<ntiles) {
				get_part_position(order[i],tile,&dx,&dy);
				tiles[tile[order[i]]]->draw_reflection(x+dx,y+dy,sfc,reflection_kh,reflection_kv,reflection_oh,reflection_ov,reflection_wave_height,center_x,center_y,factor);
			} /* if */ 
		} /* for */ 

	} /* if */ 
} /* OBJECT::draw_reflection */ 


void OBJECT::draw_refraction(int x,int y,int *tile,int *p_offset_p,SDL_Surface *sfc,SDL_Rect *vp,int refraction_distance,int refraction_kh,int refraction_kv,int refraction_oh,int refraction_ov,int refraction_wave_height,int factor)
{
	int x1,x2,y1,y2;
	int i;
	int dx,dy;
	int order[MAX_PARTS];

	if (vp!=0) get_bbox(x,y,tile,&x1,&y1,&x2,&y2);

	if (vp==0 ||
		(x2>vp->x &&
		 y2>vp->y &&
		 x1<vp->x+vp->w &&
		 y1<vp->y+vp->h)) {
		/* Draw bitmaps: */ 

		get_parts_order(order,p_offset_p);

		for(i=0;i<nparts;i++) {
			if (tile[order[i]]>=0 && tile[order[i]]<ntiles) {
				get_part_position(order[i],tile,&dx,&dy);
				tiles[tile[order[i]]]->draw_refraction(x+dx,y+dy,sfc,refraction_distance,refraction_kh,refraction_kv,refraction_oh,refraction_ov,refraction_wave_height,factor);
			} /* if */ 
		} /* for */ 

	} /* if */ 
} /* OBJECT::draw_refraction */ 


void OBJECT::draw_transform(int x,int y,int *tile,int *p_offset_p,SDL_Surface *dest,SDL_Rect *vp,float scale,float angle)
{
	int x1,x2,y1,y2;
	int i,t;
	int dx,dy;
	int order[MAX_PARTS];

	if (vp!=0) {
		get_bbox(x,y,tile,&x1,&y1,&x2,&y2);
		/* Transform de BBOX: */ 
		x1=int((x1-x)*scale);
		y1=int((y1-y)*scale);
		x2=int((x2-x)*scale);
		y2=int((y2-y)*scale);
		rotate_coordinates(x1,y1,angle,&x1,&y1);
		rotate_coordinates(x2,y2,angle,&x2,&y2);
		x1+=x;
		y1+=y;
		x2+=x;
		y2+=y;
	} /* if */ 

	if (vp==0 ||
		(x2>vp->x &&
		 y2>vp->y &&
		 x1<vp->x+vp->w &&
		 y1<vp->y+vp->h)) {
		/* Draw bitmaps: */ 

		get_parts_order(order,p_offset_p);

		for(i=0;i<nparts;i++) {
			t=tile[order[i]];
			if (t>=0 && t<ntiles) {
				get_part_position_transform(order[i],tile,scale,angle,&dx,&dy);
				tiles[t]->draw_transform(x+dx,y+dy,dest,scale,angle);
			} /* if */ 
		} /* for */ 
	} /* if */ 
} /* draw_transform */ 


void OBJECT::draw_shaded_transform(int x,int y,int *tile,int *p_offset_p,SDL_Surface *dest,SDL_Rect *vp,float scale,float angle,int factor,int r,int g,int b,int a)
{
	int x1,x2,y1,y2;
	int i,t;
	int dx,dy;
	int order[MAX_PARTS];

	if (vp!=0) {
		get_bbox(x,y,tile,&x1,&y1,&x2,&y2);
		/* Transform de BBOX: */ 
		x1=int((x1-x)*scale);
		y1=int((y1-y)*scale);
		x2=int((x2-x)*scale);
		y2=int((y2-y)*scale);
		rotate_coordinates(x1,y1,angle,&x1,&y1);
		rotate_coordinates(x2,y2,angle,&x2,&y2);
		x1+=x;
		y1+=y;
		x2+=x;
		y2+=y;
	} /* if */ 

	if (vp==0 ||
		(x2>vp->x &&
		 y2>vp->y &&
		 x1<vp->x+vp->w &&
		 y1<vp->y+vp->h)) {
		/* Draw bitmaps: */ 
		get_parts_order(order,p_offset_p);
		for(i=0;i<nparts;i++) {
			t=tile[order[i]];
			if (t>=0 && t<ntiles) {
				get_part_position_transform(order[i],tile,scale,angle,&dx,&dy);
				tiles[t]->draw_shaded_transform(x+dx,y+dy,dest,scale,angle,factor,r,g,b,a);
			} /* if */ 
		} /* for */ 
	} /* if */ 
} /* draw_shaded_transform */ 


void OBJECT::draw_bicolor_transform(int x,int y,int *tile,int *p_offset_p,SDL_Surface *dest,SDL_Rect *vp,float scale,float angle,int factor,int r1,int g1,int b1,int a1,int r2,int g2,int b2,int a2)
{
	int x1,x2,y1,y2;
	int i,t;
	int dx,dy;
	int order[MAX_PARTS];

	if (vp!=0) {
		get_bbox(x,y,tile,&x1,&y1,&x2,&y2);
		/* Transform de BBOX: */ 
		x1=int((x1-x)*scale);
		y1=int((y1-y)*scale);
		x2=int((x2-x)*scale);
		y2=int((y2-y)*scale);
		rotate_coordinates(x1,y1,angle,&x1,&y1);
		rotate_coordinates(x2,y2,angle,&x2,&y2);
		x1+=x;
		y1+=y;
		x2+=x;
		y2+=y;
	} /* if */ 

	if (vp==0 ||
		(x2>vp->x &&
		 y2>vp->y &&
		 x1<vp->x+vp->w &&
		 y1<vp->y+vp->h)) {
		/* Draw bitmaps: */ 

		get_parts_order(order,p_offset_p);

		for(i=0;i<nparts;i++) {
			t=tile[order[i]];
			if (t>=0 && t<ntiles) {
				get_part_position_transform(order[i],tile,scale,angle,&dx,&dy);
				tiles[t]->draw_bicolor_transform(x+dx,y+dy,dest,scale,angle,factor,r1,g1,b1,a1,r2,g2,b2,a2);
			} /* if */ 
		} /* for */ 
	} /* if */ 
} /* draw_bicolor_transform */ 


void OBJECT::draw_reflection_transform(int x,int y,int *tile,int *p_offset_p,SDL_Surface *dest,SDL_Rect *vp,float scale,float angle,int reflection_kh,int reflection_kv,int reflection_oh,int reflection_ov,int reflection_wave_height,int center_x,int center_y,int factor)
{
	int x1,x2,y1,y2;
	int i,t;
	int dx,dy;
	int order[MAX_PARTS];

	if (vp!=0) {
		get_bbox(x,y,tile,&x1,&y1,&x2,&y2);
		/* Transform de BBOX: */ 
		x1=int((x1-x)*scale);
		y1=int((y1-y)*scale);
		x2=int((x2-x)*scale);
		y2=int((y2-y)*scale);
		rotate_coordinates(x1,y1,angle,&x1,&y1);
		rotate_coordinates(x2,y2,angle,&x2,&y2);
		x1+=x;
		y1+=y;
		x2+=x;
		y2+=y;
	} /* if */ 

	if (vp==0 ||
		(x2>vp->x &&
		 y2>vp->y &&
		 x1<vp->x+vp->w &&
		 y1<vp->y+vp->h)) {
		/* Draw bitmaps: */ 

		for(i=0;i<nparts;i++) {
			t=tile[order[i]];
			if (t>=0 && t<ntiles) {
				get_part_position_transform(order[i],tile,scale,angle,&dx,&dy);
				tiles[t]->draw_reflection_transform(x+dx,y+dy,dest,scale,angle,reflection_kh,reflection_kv,reflection_oh,reflection_ov,reflection_wave_height,center_x,center_y,factor);
			} /* if */ 
		} /* for */ 

	} /* if */ 
} /* draw_reflection_transform */ 


void OBJECT::draw_refraction_transform(int x,int y,int *tile,int *p_offset_p,SDL_Surface *dest,SDL_Rect *vp,float scale,float angle,int refraction_distance,int refraction_kh,int refraction_kv,int refraction_oh,int refraction_ov,int refraction_wave_height,int factor)
{
	int x1,x2,y1,y2;
	int i,t;
	int dx,dy;
	int order[MAX_PARTS];

	if (vp!=0) {
		get_bbox(x,y,tile,&x1,&y1,&x2,&y2);
		/* Transform de BBOX: */ 
		x1=int((x1-x)*scale);
		y1=int((y1-y)*scale);
		x2=int((x2-x)*scale);
		y2=int((y2-y)*scale);
		rotate_coordinates(x1,y1,angle,&x1,&y1);
		rotate_coordinates(x2,y2,angle,&x2,&y2);
		x1+=x;
		y1+=y;
		x2+=x;
		y2+=y;
	} /* if */ 

	if (vp==0 ||
		(x2>vp->x &&
		 y2>vp->y &&
		 x1<vp->x+vp->w &&
		 y1<vp->y+vp->h)) {
		/* Draw bitmaps: */ 

		for(i=0;i<nparts;i++) {
			t=tile[order[i]];
			if (t>=0 && t<ntiles) {
				get_part_position_transform(order[i],tile,scale,angle,&dx,&dy);
				tiles[t]->draw_refraction_transform(x+dx,y+dy,dest,scale,angle,refraction_distance,refraction_kh,refraction_kv,refraction_oh,refraction_ov,refraction_wave_height,factor);
			} /* if */ 
		} /* for */ 

	} /* if */ 
} /* draw_refraction_transform */ 


void OBJECT::draw_collision_mask(int x,int y,int *tile,SDL_Surface *sfc,SDL_Rect *vp)
{
	int x1,x2,y1,y2;
	int i;
	int dx,dy;
	
	if (vp!=0) get_bbox(x,y,tile,&x1,&y1,&x2,&y2);

	if (vp==0 ||
		(x2>vp->x &&
		 y2>vp->y &&
		 x1<vp->x+vp->w &&
		 y1<vp->y+vp->h)) {
		/* Draw bitmaps: */ 

		for(i=0;i<nparts;i++) {
			if (tile[i]>=0 && tile[i]<ntiles) {
				get_part_position(i,tile,&dx,&dy);
				tiles[tile[i]]->draw_collision_mask(x+dx,y+dy,sfc);
			} /* if */ 
		} /* for */ 
	} /* if */ 
} /* OBJECT::draw_collision_mask */ 


void OBJECT::draw_collision_mask_transform(int x,int y,int *tile,SDL_Surface *sfc,SDL_Rect *vp,float scale,float angle)
{
	int x1,x2,y1,y2;
	int i;
	int dx,dy;
	
	if (vp!=0) {
		get_bbox(x,y,tile,&x1,&y1,&x2,&y2);
		/* Transform de BBOX: */ 
		x1=int((x1-x)*scale);
		y1=int((y1-y)*scale);
		x2=int((x2-x)*scale);
		y2=int((y2-y)*scale);
		rotate_coordinates(x1,y1,angle,&x1,&y1);
		rotate_coordinates(x2,y2,angle,&x2,&y2);
		x1+=x;
		y1+=y;
		x2+=x;
		y2+=y;
	} /* if */ 

	if (vp==0 ||
		(x2>vp->x &&
		 y2>vp->y &&
		 x1<vp->x+vp->w &&
		 y1<vp->y+vp->h)) {
		/* Draw bitmaps: */ 

		for(i=0;i<nparts;i++) {
			if (tile[i]>=0 && tile[i]<ntiles) {
				get_part_position_transform(i,tile,scale,angle,&dx,&dy);
				tiles[tile[i]]->draw_collision_mask_transform(x+dx,y+dy,sfc,scale,angle);
			} /* if */ 
		} /* for */ 
	} /* if */ 
} /* OBJECT::draw_collision_mask_transform */ 


bool OBJECT::passive(void)
{
//	if (ntiles==1 &&
//		(constitution==CONS_NONE ||
//		 (constitution!=CONS_SOLID &&
//        constitution!=CONS_SOLID_SURFACE)) &&
//		n_states==0) return true;

	if (active_flag ||
		n_states>0 ||
		n_s_states>0 ||
		control==CTRL_TEXT ||
		control==CTRL_GAMEWINDOW ||
		control==CTRL_CHARACTER) return false;
		
	return true;
} /* OBJECT::passive */ 



int OBJECT::get_dx(int *tile)
{
	if (nparts==1) {
		if (tile[0]>=0 && tile[0]<ntiles) {
			return tiles[tile[0]]->get_dx();
		} /* if */ 
		return 0;
	} else {
		int i;
		int minx=0,maxx=0;
		int cx,cy;

		for(i=0;i<nparts;i++) {
			int tdx;

			get_part_position(i,tile,&cx,&cy);

			if (tile[i]>=0 && tile[i]<ntiles) tdx=tiles[tile[i]]->get_dx();
										 else tdx=0;

			if (i==0) {
				minx=cx;
				maxx=cx+tdx;
			} else {
				if (cx<minx) minx=cx;
				if (cx+tdx>maxx) maxx=cx+tdx;
			} /* if */ 		
		} /* for */ 
		return maxx-minx;
	} /* if */ 

} /* OBJECT::get_dx */ 


int OBJECT::get_dy(int *tile)
{
	if (nparts==1) {
		if (tile[0]>=0 && tile[0]<ntiles) {
			return tiles[tile[0]]->get_dy();
		} /* if */ 
		return 0;
	} else {
		int i;
		int miny=0,maxy=0;
		int cx,cy;

		for(i=0;i<nparts;i++) {
			int tdy;

			get_part_position(i,tile,&cx,&cy);

			if (tile[i]>=0 && tile[i]<ntiles) tdy=tiles[tile[i]]->get_dy();
										 else tdy=0;

			if (i==0) {
				miny=cy;
				maxy=cy+tdy;
			} else {
				if (cy<miny) miny=cy;
				if (cy+tdy>maxy) maxy=cy+tdy;
			} /* if */ 		
		} /* for */ 
		return maxy-miny;
	} /* if */ 
} /* OBJECT::get_dy */ 


void OBJECT::get_part_position(int part,int *tile,int *x,int *y)
{
	/* Draw bitmaps: */ 
	int cx=-link_x[(tile[0]==-1 ? 0:tile[0])][0];
	int cy=-link_y[(tile[0]==-1 ? 0:tile[0])][0];
	int c=0;
	int j=part;

	if (tile[part]>=0 && tile[part]<ntiles) {
		while(j!=0 && c<MAX_PARTS) {
			if (nlinks[tile[link_part[j]]]>link_a[j]) {
				cx+=link_x[tile[link_part[j]]][link_a[j]];
				cy+=link_y[tile[link_part[j]]][link_a[j]];
			} /* if */ 
			if (nlinks[tile[j]]>link_b[j]) {
				cx-=link_x[tile[j]][link_b[j]];
				cy-=link_y[tile[j]][link_b[j]];
			} /* if */ 

			j=link_part[j];
			c++;
		} /* while */ 
	} /* if */ 

	*x=cx;
	*y=cy;
} /* OBJECT::get_part_position */ 


void OBJECT::get_part_position_transform(int part,int *tile,float scale,float angle,int *x,int *y)
{
	/* Draw bitmaps: */ 
	int cx=-link_x[(tile[0]==-1 ? 0:tile[0])][0];
	int cy=-link_y[(tile[0]==-1 ? 0:tile[0])][0];
	int c=0;
	int j=part;

	if (tile[part]>=0 && tile[part]<ntiles) {
		while(j!=0 && c<MAX_PARTS) {

			assert(tile[link_part[j]]<ntiles &&
				   tile[link_part[j]]>=0);

			if (nlinks[tile[link_part[j]]]>link_a[j]) {
				cx+=link_x[tile[link_part[j]]][link_a[j]];
				cy+=link_y[tile[link_part[j]]][link_a[j]];
			} /* if */ 
			if (nlinks[tile[j]]>link_b[j]) {
				cx-=link_x[tile[j]][link_b[j]];
				cy-=link_y[tile[j]][link_b[j]];
			} /* if */ 

			j=link_part[j];
			c++;
		} /* while */ 
	} /* if */ 

	{
		int dw,dh;
		int w=tiles[tile[part]]->get_dx();
		int h=tiles[tile[part]]->get_dy();
		int c2x,c2y;

		//  0 - compute the dimensions of the rotated tile:
		rotate_surface_size(w,h,angle,scale,&dw,&dh);	

		//  1 - compute the coordinates of the center of the tile before the transformation
		cx=(-cx)-w/2;
		cy=(-cy)-h/2;

		//  2 - compute the top-left coordinates of the rotated tile:
		rotate_coordinates(int(cx*scale),int(cy*scale),angle,&c2x,&c2y);
		*x=-(c2x+dw/2);
		*y=-(c2y+dh/2);
	} 
} /* OBJECT::get_part_position_transform */ 


void OBJECT::get_link(int *tile,int part,int n,int *lx,int *ly)
{
	if (tile[part]>=0 && tile[part]<ntiles &&
		n>=0 && n<nlinks[tile[part]]) {
		*lx=link_x[tile[part]][n];
		*ly=link_y[tile[part]][n];
	} else {
		*lx=0;
		*ly=0;
	} /* if */ 
} /* OBJECT::get_link */ 



void OBJECT::get_bbox(int x,int y,int *tile,int *x1,int *y1,int *x2,int *y2)
{
	int i;
	int px,py;
	int minx=0,miny=0,maxx=0,maxy=0;

	for(i=0;i<nparts;i++) {
		if (tile[i]>=0 && tile[i]<ntiles) {
			get_part_position(i,tile,&px,&py);
			if (i==0) {
				minx=px;
				miny=py;
				maxx=px+tiles[tile[i]]->get_dx();
				maxy=py+tiles[tile[i]]->get_dy();
			} else {
				if (px<minx) minx=px;
				if (py<miny) miny=py;
				if (px+tiles[tile[i]]->get_dx()>maxx) maxx=px+tiles[tile[i]]->get_dx();
				if (py+tiles[tile[i]]->get_dy()>maxy) maxy=py+tiles[tile[i]]->get_dy();
			} /* if */ 
		} /* if */ 
	} /* for */ 

	*x1=minx+x;
	*y1=miny+y;
	*x2=maxx+x;
	*y2=maxy+y;
} /* OBJECT::get_bbox */ 


int OBJECT::get_variable_value(char *vname,int *ival,char **sval)
{

	if (strcmp(vname,"state")==0) {
		*ival=init_state;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"scale")==0) {
		*ival=scale;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"angle")==0) {
		*ival=angle;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"energy")==0) {
		*ival=init_energy;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"experience")==0) {
		*ival=experience;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"damage")==0) {
		*ival=damage;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"damage_type")==0) {
		*sval=damage_type;
		return VALUE_STRING;
	} else if (strcmp(vname,"jumping_ctnt")==0) {
		*ival=jumping_ctnt;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"gravity_ctnt")==0) {
		*ival=gravity_ctnt;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"walking_speed")==0) {
		*ival=walking_speed;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"ladder_up_speed")==0) {
		*ival=ladder_up_speed;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"ladder_down_speed")==0) {
		*ival=ladder_down_speed;
		return VALUE_INTEGER;

	} else if (strcmp(vname,"dm_drawing_mode")==0) {
		*ival=dm_drawing_mode;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_color1_r")==0) {
		*ival=dm_color1_r;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_color1_g")==0) {
		*ival=dm_color1_g;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_color1_b")==0) {
		*ival=dm_color1_b;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_color1_a")==0) {
		*ival=dm_color1_a;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_color2_r")==0) {
		*ival=dm_color2_r;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_color2_g")==0) {
		*ival=dm_color2_g;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_color2_b")==0) {
		*ival=dm_color2_b;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_color2_a")==0) {
		*ival=dm_color2_a;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_factor")==0) {
		*ival=dm_factor;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_blink_f0")==0) {
		*ival=dm_blink_f0;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_blink_f1")==0) {
		*ival=dm_blink_f1;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_blink_speed")==0) {
		*ival=dm_blink_speed;
		return VALUE_INTEGER;

	} else if (strcmp(vname,"text_text")==0) {
		*sval=text_text;
		return VALUE_STRING;
	} else if (strcmp(vname,"text_font")==0) {
		*sval=text_font;
		return VALUE_STRING;
	} else if (strcmp(vname,"text_size")==0) {
		*ival=text_size;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_spacing")==0) {
		*ival=text_spacing;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_h_align")==0) {
		*ival=text_h_align;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_v_align")==0) {
		*ival=text_v_align;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_r")==0) {
		*ival=text_r;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_g")==0) {
		*ival=text_g;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_b")==0) {
		*ival=text_b;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_a")==0) {
		*ival=text_a;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame")==0) {
		*ival=text_frame;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_point_x")==0) {
		*ival=text_frame_point_x;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_point_y")==0) {
		*ival=text_frame_point_y;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_r1")==0) {
		*ival=text_frame_r1;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_g1")==0) {
		*ival=text_frame_g1;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_b1")==0) {
		*ival=text_frame_b1;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_a1")==0) {
		*ival=text_frame_a1;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_r2")==0) {
		*ival=text_frame_r2;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_g2")==0) {
		*ival=text_frame_g2;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_b2")==0) {
		*ival=text_frame_b2;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_a2")==0) {
		*ival=text_frame_a2;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_width")==0) {
		*ival=text_frame_width;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_radius")==0) {
		*ival=text_frame_radius;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_max_w")==0) {
		*ival=text_max_w;
		return VALUE_INTEGER;

	} else if (strcmp(vname,"gw_dx")==0) {
		*ival=gw_dx;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"gw_dy")==0) {
		*ival=gw_dy;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"gw_scale")==0) {
		*ival=gw_scale;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"gw_min_hs_x")==0) {
		*ival=gw_min_hs_x;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"gw_min_hs_y")==0) {
		*ival=gw_min_hs_y;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"gw_max_hs_x")==0) {
		*ival=gw_max_hs_x;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"gw_max_hs_y")==0) {
		*ival=gw_max_hs_y;
		return VALUE_INTEGER;

	} else if (strcmp(vname,"menu_dx")==0) {
		*ival=menu_dx;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"menu_dy")==0) {
		*ival=menu_dy;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"menu_title")==0) {
		*sval=menu_title;
		return VALUE_STRING;
	} else if (strcmp(vname,"menu_appearance_flags")==0) {
		*ival=menu_appearance_flags;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"menu_entry_effect_flags")==0) {
		*ival=menu_entry_effect_flags;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"menu_blink_timmer")==0) {
		*ival=menu_blink_timmer;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"menu_zoom_factor")==0) {
		*ival=menu_zoom_factor;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"menu_appear_factor")==0) {
		*ival=menu_appear_factor;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"menu_text_r2")==0) {
		*ival=menu_text_r2;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"menu_text_g2")==0) {
		*ival=menu_text_g2;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"menu_text_b2")==0) {
		*ival=menu_text_b2;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"menu_text_a2")==0) {
		*ival=menu_text_a2;
		return VALUE_INTEGER;

	} else {
		List<VARIABLE> l;
		VARIABLE *v;

		l.Instance(object_variables);
		l.Rewind();
		while(l.Iterate(v)) {
			if (v->cmp(vname)) return v->get(ival,sval,0);
		} /* while */ 
		return VALUE_NONE;
	} /* if */ 

} /* OBJECT::get_variable_value */ 


void OBJECT::set_variable_value(char *vname,int value)
{
	if (strcmp(vname,"state")==0) {
		init_state=value;
	} else if (strcmp(vname,"scale")==0) {
		scale=value;
	} else if (strcmp(vname,"angle")==0) {
		angle=value;
	} else if (strcmp(vname,"energy")==0) {
		init_energy=value;
	} else if (strcmp(vname,"experience")==0) {
		experience=value;
	} else if (strcmp(vname,"damage")==0) {
		damage=value;
	} else if (strcmp(vname,"damage_type")==0) {
	} else if (strcmp(vname,"jumping_ctnt")==0) {
		jumping_ctnt=value;
	} else if (strcmp(vname,"gravity_ctnt")==0) {
		gravity_ctnt=value;
	} else if (strcmp(vname,"walking_speed")==0) {
		walking_speed=value;
	} else if (strcmp(vname,"ladder_up_speed")==0) {
		ladder_up_speed=value;
	} else if (strcmp(vname,"ladder_down_speed")==0) {
		ladder_down_speed=value;
	} else if (strcmp(vname,"dm_drawing_mode")==0) {
		dm_drawing_mode=value;
	} else if (strcmp(vname,"dm_color1_r")==0) {
		dm_color1_r=value;
	} else if (strcmp(vname,"dm_color1_g")==0) {
		dm_color1_g=value;
	} else if (strcmp(vname,"dm_color1_b")==0) {
		dm_color1_b=value;
	} else if (strcmp(vname,"dm_color1_a")==0) {
		dm_color1_a=value;
	} else if (strcmp(vname,"dm_color2_r")==0) {
		dm_color2_r=value;
	} else if (strcmp(vname,"dm_color2_g")==0) {
		dm_color2_g=value;
	} else if (strcmp(vname,"dm_color2_b")==0) {
		dm_color2_b=value;
	} else if (strcmp(vname,"dm_color2_a")==0) {
		dm_color2_a=value;
	} else if (strcmp(vname,"dm_factor")==0) {
		dm_factor=value;
	} else if (strcmp(vname,"dm_blink_f0")==0) {
		dm_blink_f0=value;
	} else if (strcmp(vname,"dm_blink_f1")==0) {
		dm_blink_f1=value;
	} else if (strcmp(vname,"dm_blink_speed")==0) {
		dm_blink_speed=value;
	} else if (strcmp(vname,"text_size")==0) {
		text_size=value;
	} else if (strcmp(vname,"text_spacing")==0) {
		text_spacing=value;
	} else if (strcmp(vname,"text_h_align")==0) {
		text_h_align=value;
	} else if (strcmp(vname,"text_v_align")==0) {
		text_v_align=value;
	} else if (strcmp(vname,"text_r")==0) {
		text_r=value;
	} else if (strcmp(vname,"text_g")==0) {
		text_g=value;
	} else if (strcmp(vname,"text_b")==0) {
		text_b=value;
	} else if (strcmp(vname,"text_a")==0) {
		text_a=value;
	} else if (strcmp(vname,"text_frame")==0) {
		text_frame=value;
	} else if (strcmp(vname,"text_frame_point_x")==0) {
		text_frame_point_x=value;
	} else if (strcmp(vname,"text_frame_point_y")==0) {
		text_frame_point_y=value;
	} else if (strcmp(vname,"text_frame_r1")==0) {
		text_frame_r1=value;
	} else if (strcmp(vname,"text_frame_g1")==0) {
		text_frame_g1=value;
	} else if (strcmp(vname,"text_frame_b1")==0) {
		text_frame_b1=value;
	} else if (strcmp(vname,"text_frame_a1")==0) {
		text_frame_a1=value;
	} else if (strcmp(vname,"text_frame_r2")==0) {
		text_frame_r2=value;
	} else if (strcmp(vname,"text_frame_g2")==0) {
		text_frame_g2=value;
	} else if (strcmp(vname,"text_frame_b2")==0) {
		text_frame_b2=value;
	} else if (strcmp(vname,"text_frame_a2")==0) {
		text_frame_a2=value;
	} else if (strcmp(vname,"text_frame_width")==0) {
		text_frame_width=value;
	} else if (strcmp(vname,"text_frame_radius")==0) {
		text_frame_radius=value;
	} else if (strcmp(vname,"text_max_w")==0) {
		text_max_w=value;

	} else if (strcmp(vname,"gw_dx")==0) {
		gw_dx=value;
	} else if (strcmp(vname,"gw_dy")==0) {
		gw_dy=value;
	} else if (strcmp(vname,"gw_scale")==0) {
		gw_scale=value;
	} else if (strcmp(vname,"gw_min_hs_x")==0) {
		gw_min_hs_x=value;
	} else if (strcmp(vname,"gw_min_hs_y")==0) {
		gw_min_hs_y=value;
	} else if (strcmp(vname,"gw_max_hs_x")==0) {
		gw_max_hs_x=value;
	} else if (strcmp(vname,"gw_max_hs_y")==0) {
		gw_max_hs_y=value;

	} else if (strcmp(vname,"menu_dx")==0) {
		menu_dx=value;
	} else if (strcmp(vname,"menu_dy")==0) {
		menu_dy=value;
	} else if (strcmp(vname,"menu_appearance_flags")==0) {
		menu_appearance_flags=value;
	} else if (strcmp(vname,"menu_entry_effect_flags")==0) {
		menu_entry_effect_flags=value;
	} else if (strcmp(vname,"menu_blink_timmer")==0) {
		menu_blink_timmer=value;
	} else if (strcmp(vname,"menu_zoom_factor")==0) {
		menu_zoom_factor=value;
	} else if (strcmp(vname,"menu_appear_factor")==0) {
		menu_appear_factor=value;
	} else if (strcmp(vname,"menu_text_r2")==0) {
		menu_text_r2=value;
	} else if (strcmp(vname,"menu_text_g2")==0) {
		menu_text_g2=value;
	} else if (strcmp(vname,"menu_text_b2")==0) {
		menu_text_b2=value;
	} else if (strcmp(vname,"menu_text_a2")==0) {
		menu_text_a2=value;

	} else {
		List<VARIABLE> l;
		VARIABLE *v;

		l.Instance(object_variables);
		l.Rewind();
		while(l.Iterate(v)) {
			if (v->cmp(vname)) {
				v->set(value);
				return;
			} /* if */ 
		} /* while */ 
	} /* if */ 
} /* OBJECT::set_variable_value */ 


void OBJECT::set_variable_value(char *vname,char *value)
{
	if (strcmp(vname,"damage_type")==0) {
		delete damage_type;
		damage_type=new char[strlen(value)+1];
		strcpy(damage_type,value);
	} else if (strcmp(vname,"text_text")==0) {
		delete []text_text;
		text_text=new char[strlen(value)+1];
		strcpy(text_text,value);
	} else if (strcmp(vname,"text_font")==0) {
		delete []text_font;
		text_font=new char[strlen(value)+1];
		strcpy(text_font,value);

	} else if (strcmp(vname,"menu_title")==0) {
		delete []menu_title;
		menu_title=new char[strlen(value)+1];
		strcpy(menu_title,value);
	} else {

		List<VARIABLE> l;
		VARIABLE *v;

		l.Instance(object_variables);
		l.Rewind();
		while(l.Iterate(v)) {
			if (v->cmp(vname)) {
				v->set(value);
				return;
			} /* if */ 
		} /* while */ 
	} /* if */ 
} /* OBJECT::set_variable_value */ 


void OBJECT::get_parts_order(int *order,int *p_offset_p)
{
	int i;

	if (nparts>1) {
		int tmp;
		bool change=false;
		/* Sort for precedences: */ 
		for(i=0;i<nparts;i++) order[i]=i;

		if (p_offset_p==0) {
			do{
				change=false;
				for(i=0;i<nparts-1;i++) {
					if (part_precedence[order[i]]>part_precedence[order[i+1]]) {
						tmp=order[i];
						order[i]=order[i+1];
						order[i+1]=tmp;
						change=true;
					} /* if */ 
				} /* for */ 
			}while(change);
		} else {
			do{
				change=false;
				for(i=0;i<nparts-1;i++) {
					if (part_precedence[order[i]]+p_offset_p[order[i]] >
						part_precedence[order[i+1]]+p_offset_p[order[i+1]]) {
						tmp=order[i];
						order[i]=order[i+1];
						order[i+1]=tmp;
						change=true;
					} /* if */ 
				} /* for */ 
			}while(change);
		} /* if */ 
	} else {
		order[0]=0;
	} /* if */ 

} /* OBJECT::get_parts_order */ 



void OBJECT::gw_draw(int x,int y,SDL_Surface *sfc,SDL_Rect *vp)
{
	if (vp!=0) draw_rectangle_vp(sfc,x,y,gw_dx,gw_dy,SDL_MapRGB(sfc->format,255,255,255),vp);
		  else draw_rectangle(sfc,x,y,gw_dx,gw_dy,SDL_MapRGB(sfc->format,255,255,255));
} /* OBJECT::gw_draw */ 


void OBJECT::menu_draw(int x,int y,SDL_Surface *sfc,SDL_Rect *vp)
{
	if (vp!=0) draw_rectangle_vp(sfc,x,y,menu_dx,menu_dy,SDL_MapRGB(sfc->format,255,255,255),vp);
		  else draw_rectangle(sfc,x,y,menu_dx,menu_dy,SDL_MapRGB(sfc->format,255,255,255));
} /* OBJECT::menu_draw */ 



void OBJECT::get_gw_bbox(int x,int y,int *x1,int *y1,int *x2,int *y2)
{
	*x1=x;
	*y1=x;
	*x2=x+gw_dx;
	*y2=y+gw_dy;
} /* OBJECT::get_gw_bbox */ 


void OBJECT::get_menu_bbox(int x,int y,int *x1,int *y1,int *x2,int *y2)
{
	*x1=x;
	*y1=x;
	*x2=x+menu_dx;
	*y2=y+menu_dy;
} /* OBJECT::get_menu_bbox */ 



bool object_order(OBJECT *o1,OBJECT *o2)
{
	if (strcmp(o1->name,o2->name)<0) return true;

	return false;
} /* object_order */ 


void OBJECT::get_variable_names(List<char> *variable_names)
{
	char *tmp;
	List<VARIABLE> lv;
	VARIABLE *v;

	tmp=new char[6];
	strcpy(tmp,"state");
	variable_names->Add(tmp);

	tmp=new char[5];
	strcpy(tmp,"this");
	variable_names->Add(tmp);

	tmp=new char[2];
	strcpy(tmp,"x");
	variable_names->Add(tmp);

	tmp=new char[2];
	strcpy(tmp,"y");
	variable_names->Add(tmp);

	tmp=new char[6];
	strcpy(tmp,"scale");
	variable_names->Add(tmp);

	tmp=new char[6];
	strcpy(tmp,"angle");
	variable_names->Add(tmp);

	tmp=new char[7];
	strcpy(tmp,"energy");
	variable_names->Add(tmp);

	tmp=new char[11];
	strcpy(tmp,"experience");
	variable_names->Add(tmp);

	tmp=new char[20];
	strcpy(tmp,"relative_precedence");
	variable_names->Add(tmp);

	tmp=new char[16];
	strcpy(tmp,"dm_drawing_mode");
	variable_names->Add(tmp);

	tmp=new char[12];
	strcpy(tmp,"dm_color1_r");
	variable_names->Add(tmp);

	tmp=new char[12];
	strcpy(tmp,"dm_color1_g");
	variable_names->Add(tmp);

	tmp=new char[12];
	strcpy(tmp,"dm_color1_b");
	variable_names->Add(tmp);

	tmp=new char[12];
	strcpy(tmp,"dm_color1_a");
	variable_names->Add(tmp);

	tmp=new char[12];
	strcpy(tmp,"dm_color2_r");
	variable_names->Add(tmp);

	tmp=new char[12];
	strcpy(tmp,"dm_color2_g");
	variable_names->Add(tmp);

	tmp=new char[12];
	strcpy(tmp,"dm_color2_b");
	variable_names->Add(tmp);

	tmp=new char[12];
	strcpy(tmp,"dm_color2_a");
	variable_names->Add(tmp);

	tmp=new char[10];
	strcpy(tmp,"dm_factor");
	variable_names->Add(tmp);

	tmp=new char[12];
	strcpy(tmp,"dm_blink_f0");
	variable_names->Add(tmp);

	tmp=new char[12];
	strcpy(tmp,"dm_blink_f1");
	variable_names->Add(tmp);

	tmp=new char[15];
	strcpy(tmp,"dm_blink_speed");
	variable_names->Add(tmp);

	tmp=new char[13];
	strcpy(tmp,"object_plane");
	variable_names->Add(tmp);

	switch(constitution) {
	case CONS_NONE: 
					break;
	case CONS_SOLID:
 					break;
	case CONS_SOLID_SURFACE:
 					break;
	case CONS_WEAPON:
				tmp=new char[7];
				strcpy(tmp,"damage");
				variable_names->Add(tmp);

				tmp=new char[12];
				strcpy(tmp,"damage_type");
				variable_names->Add(tmp);
			 
				break;
	case CONS_LADDER:
					 						
					break;
	case CONS_CHARACTER: 
					break;
	} /* switch */ 

	switch(control) {
	case CTRL_STANDARD:
		break;
	case CTRL_CHARACTER:
					tmp=new char[7];
					strcpy(tmp,"key_up");
					variable_names->Add(tmp);

					tmp=new char[9];
					strcpy(tmp,"key_down");
					variable_names->Add(tmp);

					tmp=new char[9];
					strcpy(tmp,"key_left");
					variable_names->Add(tmp);

					tmp=new char[10];
					strcpy(tmp,"key_right");
					variable_names->Add(tmp);

					tmp=new char[10];
					strcpy(tmp,"key_fire1");
					variable_names->Add(tmp);

					tmp=new char[10];
					strcpy(tmp,"key_fire2");
					variable_names->Add(tmp);

					tmp=new char[10];
					strcpy(tmp,"key_fire3");
					variable_names->Add(tmp);

					tmp=new char[10];
					strcpy(tmp,"key_fire4");
					variable_names->Add(tmp);

					tmp=new char[13];
					strcpy(tmp,"jumping_ctnt");
					variable_names->Add(tmp);

					tmp=new char[13];
					strcpy(tmp,"gravity_ctnt");
					variable_names->Add(tmp);

					tmp=new char[14];
					strcpy(tmp,"walking_speed");
					variable_names->Add(tmp);

					tmp=new char[16];
					strcpy(tmp,"ladder_up_speed");
					variable_names->Add(tmp);

					tmp=new char[18];
					strcpy(tmp,"ladder_down_speed");
					variable_names->Add(tmp);

		break;
	case CTRL_TEXT:
				tmp=new char[10];
				strcpy(tmp,"text_text");
				variable_names->Add(tmp);

				tmp=new char[10];
				strcpy(tmp,"text_font");
				variable_names->Add(tmp);

				tmp=new char[10];
				strcpy(tmp,"text_size");
				variable_names->Add(tmp);

				tmp=new char[13];
				strcpy(tmp,"text_spacing");
				variable_names->Add(tmp);

				tmp=new char[7];
				strcpy(tmp,"text_r");
				variable_names->Add(tmp);

				tmp=new char[7];
				strcpy(tmp,"text_g");
				variable_names->Add(tmp);

				tmp=new char[7];
				strcpy(tmp,"text_b");
				variable_names->Add(tmp);

				tmp=new char[7];
				strcpy(tmp,"text_a");
				variable_names->Add(tmp);

				tmp=new char[13];
				strcpy(tmp,"text_h_align");
				variable_names->Add(tmp);

				tmp=new char[13];
				strcpy(tmp,"text_v_align");
				variable_names->Add(tmp);

				tmp=new char[11];
				strcpy(tmp,"text_frame");
				variable_names->Add(tmp);

				tmp=new char[19];
				strcpy(tmp,"text_frame_point_x");
				variable_names->Add(tmp);
				
				tmp=new char[19];
				strcpy(tmp,"text_frame_point_y");
				variable_names->Add(tmp);

				tmp=new char[14];
				strcpy(tmp,"text_frame_r1");
				variable_names->Add(tmp);
				
				tmp=new char[14];
				strcpy(tmp,"text_frame_g1");
				variable_names->Add(tmp);
				
				tmp=new char[14];
				strcpy(tmp,"text_frame_b1");
				variable_names->Add(tmp);
				
				tmp=new char[14];
				strcpy(tmp,"text_frame_a1");
				variable_names->Add(tmp);
				
				tmp=new char[14];
				strcpy(tmp,"text_frame_r2");
				variable_names->Add(tmp);
				
				tmp=new char[14];
				strcpy(tmp,"text_frame_g2");
				variable_names->Add(tmp);
				
				tmp=new char[14];
				strcpy(tmp,"text_frame_b2");
				variable_names->Add(tmp);
				
				tmp=new char[14];
				strcpy(tmp,"text_frame_a2");
				variable_names->Add(tmp);
				
				tmp=new char[17];
				strcpy(tmp,"text_frame_width");
				variable_names->Add(tmp);

				tmp=new char[18];
				strcpy(tmp,"text_frame_radius");
				variable_names->Add(tmp);

				tmp=new char[17];
				strcpy(tmp,"text_max_w");
				variable_names->Add(tmp);
		break;
	case CTRL_GAMEWINDOW:
				tmp=new char[12];
				strcpy(tmp,"gw_focusing");
				variable_names->Add(tmp);	

				tmp=new char[14];
				strcpy(tmp,"gw_focusing_x");
				variable_names->Add(tmp);	

				tmp=new char[14];
				strcpy(tmp,"gw_focusing_y");
				variable_names->Add(tmp);	

				tmp=new char[6];
				strcpy(tmp,"gw_dx");
				variable_names->Add(tmp);

				tmp=new char[6];
				strcpy(tmp,"gw_dy");
				variable_names->Add(tmp);

				tmp=new char[9];
				strcpy(tmp,"gw_scale");
				variable_names->Add(tmp);

				tmp=new char[12];
				strcpy(tmp,"gw_min_hs_x");
				variable_names->Add(tmp);

				tmp=new char[12];
				strcpy(tmp,"gw_min_hs_y");
				variable_names->Add(tmp);

				tmp=new char[12];
				strcpy(tmp,"gw_max_hs_x");
				variable_names->Add(tmp);

				tmp=new char[12];
				strcpy(tmp,"gw_max_hs_y");
				variable_names->Add(tmp);
		break;
	case CTRL_MENU:
				tmp=new char[8];
				strcpy(tmp,"menu_dx");
				variable_names->Add(tmp);

				tmp=new char[8];
				strcpy(tmp,"menu_dy");
				variable_names->Add(tmp);

				tmp=new char[11];
				strcpy(tmp,"menu_title");
				variable_names->Add(tmp);

				tmp=new char[22];
				strcpy(tmp,"menu_appearance_flags");
				variable_names->Add(tmp);

				tmp=new char[24];
				strcpy(tmp,"menu_entry_effect_flags");
				variable_names->Add(tmp);

				tmp=new char[18];
				strcpy(tmp,"menu_blink_timmer");
				variable_names->Add(tmp);

				tmp=new char[17];
				strcpy(tmp,"menu_zoom_factor");
				variable_names->Add(tmp);

				tmp=new char[19];
				strcpy(tmp,"menu_appear_factor");
				variable_names->Add(tmp);

				tmp=new char[10];
				strcpy(tmp,"text_font");
				variable_names->Add(tmp);

				tmp=new char[10];
				strcpy(tmp,"text_size");
				variable_names->Add(tmp);

				tmp=new char[13];
				strcpy(tmp,"text_spacing");
				variable_names->Add(tmp);

				tmp=new char[7];
				strcpy(tmp,"text_r");
				variable_names->Add(tmp);

				tmp=new char[7];
				strcpy(tmp,"text_g");
				variable_names->Add(tmp);

				tmp=new char[7];
				strcpy(tmp,"text_b");
				variable_names->Add(tmp);

				tmp=new char[7];
				strcpy(tmp,"text_a");
				variable_names->Add(tmp);

				tmp=new char[13];
				strcpy(tmp,"menu_text_r2");
				variable_names->Add(tmp);

				tmp=new char[13];
				strcpy(tmp,"menu_text_g2");
				variable_names->Add(tmp);

				tmp=new char[13];
				strcpy(tmp,"menu_text_b2");
				variable_names->Add(tmp);

				tmp=new char[13];
				strcpy(tmp,"menu_text_a2");
				variable_names->Add(tmp);

				tmp=new char[13];
				strcpy(tmp,"text_h_align");
				variable_names->Add(tmp);

				tmp=new char[12];
				strcpy(tmp,"menu_active");
				variable_names->Add(tmp);

				tmp=new char[12];
				strcpy(tmp,"menu_timmer");
				variable_names->Add(tmp);

				tmp=new char[14];
				strcpy(tmp,"menu_selected");
				variable_names->Add(tmp);
			break;
	} /* switch */ 

	/* Temporal variables: */ 
	lv.Instance(object_variables);
	lv.Rewind();
	while(lv.Iterate(v)) {
		tmp=new char[strlen(v->get_name())+1];
		strcpy(tmp,v->get_name());
		variable_names->Add(tmp);
	} /* while */ 
} /* OBJECT::get_variable_names */ 
